% Implements the thresholding of shearlet coefficients for different kinds
% of objects (phase, amplitude, mixed and real objects) with general
% thresholding functions specified in the retrieveProximityName function.
%
% Input:    input_data  - General input_data structure.
%           Psi         - Object to be thresholded.
%       
% Output:   outputPsi   - Object after thresholding of shearlet
%                           coefficients.
%

function outputPsi = proxShearlet(input_data, Psi, gamma)

shrinkageOperator = retrieveShrinkageName(input_data);

if strcmp(input_data.problem.type, 'phase')
    
    PsiPhase                = angle(Psi);

    if isfield(input_data.problem.constraint, 'range')
        if strcmp(input_data.problem.constraint.range, 'positive')
            outputPsiPhase = max(PsiPhase, 0);
        else        
            outputPsiPhase = min(PsiPhase, 0);
        end
    else
            outputPsiPhase = min(PsiPhase, 0);
    end
    
    
    %coefficientsPhase       = SLsheardec2D(PsiPhase,input_data.system);
    coefficientsPhase       = SLsheardec2D(outputPsiPhase,input_data.system);
    coefficientsPhase       = feval(shrinkageOperator, input_data, coefficientsPhase, gamma);
    
    outputPsiPhase          = SLshearrec2D(coefficientsPhase, input_data.system);
    
%     if strcmp(input_data.problem.constraint.name, 'shearlet thresholding and range')
%         % if no range type is specified, we go with negativity for phase
%         % objects.
%         if isfield(input_data.problem.constraint, 'range')
%             if strcmp(input_data.problem.constraint.range, 'positive')
%                 outputPsiPhase = max(outputPsiPhase, 0);
%             else        
%                 outputPsiPhase = min(outputPsiPhase, 0);
%             end
%         else
%                 outputPsiPhase = min(outputPsiPhase, 0);
%         end
%         outputPsiPhase      = min(outputPsiPhase, 0);
%     end
    
    outputPsi               = exp(1i * outputPsiPhase);

elseif strcmp(input_data.problem.type, 'amplitude')
    PsiAmplitude            = log(real(Psi));

    coefficientsAmplitude   = SLsheardec2D(PsiAmplitude,input_data.system);   
    coefficientsAmplitude   = feval(shrinkageOperator, input_data, coefficientsAmplitude, gamma);
      
    outputPsiAmplitude      = SLshearrec2D(coefficientsAmplitude, input_data.system);
    
    if strcmp(input_data.problem.constraint.name, 'shearlet thresholding and range')
        outputPsiAmplitude   = min(outputPsiAmplitude, 1); 
    end							

    outputPsi               = exp(outputPsiAmplitude);
    
elseif strcmp(input_data.problem.type, 'mixed')
    PsiPhase                = angle(Psi);
    PsiAmplitude            = abs(Psi);
    
    coefficientsPhase       = SLsheardec2D(PsiPhase,input_data.system);
    coefficientsAmplitude   = SLsheardec2D(PsiAmplitude,input_data.system);   

    coefficientsPhase       = feval(shrinkageOperator, input_data, coefficientsPhase, gamma);
    coefficientsAmplitude   = feval(shrinkageOperator, input_data, coefficientsAmplitude, gamma);
    
    outputPsiPhase          = SLshearrec2D(coefficientsPhase, input_data.system);
    outputPsiAmplitude      = SLshearrec2D(coefficientsAmplitude, input_data.system);
    
    if strcmp(input_data.problem.constraint.name, 'shearlet thresholding and range')
        outputPsiPhase      = min(outputPsiPhase, 0);
        outputPsiAmplitude  = min(outputPsiAmplitude, 1);
    end   
    
    outputPsi               = outputPsiAmplitude .* exp(1i * outputPsiPhase);
    
elseif strcmp(input_data.problem.type, 'real')
    PsiReal                 = real(Psi);
    
    coefficientsReal        = SLsheardec2D(PsiReal,input_data.system);
    coefficientsReal        = feval(shrinkageOperator, input_data, coefficientsReal, gamma);
    
    outputPsi               = SLshearrec2D(coefficientsReal, input_data.system);
    
    if strcmp(input_data.problem.constraint.name, 'shearlet thresholding and range')
        outputPsi           = max(real(outputPsi), 0);
    end

else error('No suitable problem type specified in input_data.problem.type.')
end 
end